local CHARACTER_DELIMITOR = " " .. ACE_TEXT_OF .. " " -- surrounding spaces are needed

MyBagsCacheClass = AceAddonClass:new({
	name = "MyBagsCache",
	description = "Caches Equipment, Inventory and Bank items",
	version = "0.6.0-tg", 
	releaseDate = "10/14/08",
	category = "inventory",
	author = "Ramble (modified by Isharra) (updated by thegabbert)",
	aceCompatible = "103",

	db = AceDbClass:new("MyBagsCacheDB"),
})
ace:RegisterFunctions(MyBagsCacheClass, {
	GetItemInfoFromLink = function(l)
		if(not l) then return end
		local _,_,c,id,il,n=strfind(l,"|cff(%x+)|Hitem:(%-?%d+)([^|]+)|h%[(.-)%]|h|r")
		return n,c,id..il,id
	end,
	SplitString = function(s,p,n)
		if (type(s) ~= "string") then
			error("SplitString must be passed a string as the first argument", 2)
		end
		
		local l,sp,ep = {},0
		while(sp) do
			sp,ep=strfind(s,p)
			if(sp) then
				tinsert(l,strsub(s,1,sp-1))
				s=strsub(s,ep+1)
			else
				tinsert(l,s)
				break
			end
			if(n) then n=n-1 end
			if(n and (n==0)) then tinsert(l,s) break end
		end
		return unpack(l)
	end,
})
function MyBagsCacheClass:Initialize()
end
function MyBagsCacheClass:Enable()
	self:RegisterEvent("PLAYER_MONEY")
	self:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
	self:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_CLOSED")
	if not self.db:get({ace.char.id},0) then 
		self:SaveItems() 
	end
end
function MyBagsCacheClass:PLAYER_MONEY()
	self.db:set({ace.char.id}, "cash", (GetMoney()))
end
function MyBagsCacheClass:PLAYERBANKBAGSLOTS_CHANGED()
	local numSlots,full = GetNumBankSlots()
	local cost = GetBankSlotCost(numSlots)
	self.db:set({ace.char.id}, "BankSlots",  {
         ["Number"] = numSlots,
         ["Cost"] = cost,
         ["Full"] = full
      })
end
function MyBagsCacheClass:PLAYERBANKSLOTS_CHANGED()
	for itemIndex = 1, 28 do
		self:SaveItemInfo(BANK_CONTAINER, itemIndex)	
	end
end
function MyBagsCacheClass:BAG_UPDATE()
	local bagIndex = arg1
	if not self.atBank and bagIndex > 4 and bagIndex <= 11 then return end
	local slots = self:SaveBagInfo(bagIndex)
	for itemIndex = 1, slots do
		self:SaveItemInfo(bagIndex, itemIndex)
	end
end
function MyBagsCacheClass:UNIT_INVENTORY_CHANGED()
	if (arg1 == "player") then
		self:SaveEquipment()
	end
end
function MyBagsCacheClass:BANKFRAME_OPENED()
	self.atBank = TRUE
	self:SaveItems()
end
function MyBagsCacheClass:BANKFRAME_CLOSED()
	self.atBank = FALSE
end
function MyBagsCacheClass:GetInfo(bagIndex, slotIndex, charID)
	local data
	slotIndex = ace.tonum(slotIndex)
	data = self.db:get({charID, bagIndex}, slotIndex)
	if data then
		return data.Texture, data.Count, data.Link, data.Color, data.Name
	end
	return nil, 0, nil, nil, nil
end
function MyBagsCacheClass:GetRelic(charID)
	local data = self.db:get({charID, "equipment"}, "hasRelic")
	if data then return data end
	return nil
end
function MyBagsCacheClass:GetCash(charID)
	local data = self.db:get({charID}, "cash")
	if data then return data end
	return nil
end
function MyBagsCacheClass:GetCharList(realm)
	local result = {}
	local cache = self.db:get()
	for index, value in pairs( cache ) do
		local charName, realmID = ace.SplitString(index, CHARACTER_DELIMITOR)
		if index ~= "profiles" then
			if (not realm or realmID == realm) then
				result[index] = {
					name = charName,
					realm = realmID,
				}
			end
		end
	end
	return result
end
function MyBagsCacheClass:GetAmmoSlotTooltipInfo()
	MyBagsCacheHiddenTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	MyBagsCacheHiddenTooltip:SetInventoryItem("player", 0)
	local field = getglobal("MyBagsCacheHiddenTooltipTextLeft1")
	local myName = field:GetText()
	local r,g,b,myColor,myLink
	if myName then 
		r,g,b = field:GetTextColor()
		myColor = string.format("%x%x%x",r*256,g*256,b*256)
	end
	MyBagsCacheHiddenTooltip:Hide()
	return myName, myColor
end 
function MyBagsCacheClass:SaveItems()
	self:SaveInventory()
	self:SaveBank()
	self:SaveEquipment()
	self:PLAYER_MONEY()
end
function MyBagsCacheClass:SaveInventory()
	local slots
	slots = self:SaveBagInfo(KEYRING_CONTAINER)
	for itemIndex = 1, slots do
		self:SaveItemInfo(KEYRING_CONTAINER, itemIndex)
	end
	for bagIndex = 0, 4 do
		slots = self:SaveBagInfo(bagIndex)
		for itemIndex = 1, slots do
			self:SaveItemInfo(bagIndex, itemIndex)
		end
	end
end
function MyBagsCacheClass:SaveBank()
	if not self.atBank then return end
	self.db:set({ace.char.id, BANK_CONTAINER, 0}, "Count",  28)
	for itemIndex = 1, 28 do
		self:SaveItemInfo(BANK_CONTAINER, itemIndex)	
	end
	for bagIndex = 5, 11 do
		local slots = self:SaveBagInfo(bagIndex)
		for itemIndex = 1, slots do
			self:SaveItemInfo(bagIndex, itemIndex)
		end
	end
end
function MyBagsCacheClass:SaveEquipment()
	for itemIndex = 0, 19 do
		self:SaveEquipmentInfo(itemIndex)
	end
	local hasRelic = UnitHasRelicSlot("player")
	self.db:set({ace.char.id, "equipment"}, "hasRelic", hasRelic)
end
function MyBagsCacheClass:SaveBagInfo(bagIndex)
	local invID -- get Inventory ID
	if bagIndex >= 1 and bagIndex <=4 then 
		invID = ContainerIDToInventoryID(bagIndex)
	elseif bagIndex >= 5 and bagIndex <= 11 then
		invID = BankButtonIDToInvSlotID(bagIndex,1)
	else
		invID = nil;
	end
	local bagSize
   if bagIndex == 0 then  -- Set Count to 16
      self.db:set({ace.char.id, bagIndex}, 0, {
         ["Count"] = 16,
         ["Texture"] = "Interface\\Buttons\\Button-Backpack-Up"
      })
      return 16
   elseif(bagIndex == KEYRING_CONTAINER) then
		bagSize = GetKeyRingSize()
		local itemLink = GetInventoryItemLink("player", invID)
		self.db:set({ace.char.id, bagIndex}, 0, {
			["Count"] = bagSize,
		})
		return bagSize
   end
	bagSize = GetContainerNumSlots(bagIndex)
	local itemLink = GetInventoryItemLink("player", invID)
	if itemLink then
		local myColor, myLink, name;
		name, myColor, myLink = ace.GetItemInfoFromLink(itemLink)
		local soulbound, madeBy = nil
		local texture = GetInventoryItemTexture("player", invID)
		self.db:set({ace.char.id, bagIndex}, 0, {
			["Name"] = name,
			["Color"] = myColor,
			["Link"] = myLink,
			["Count"] = bagSize,
			["Texture"] = texture,
			["Soulbound"] = soulbound,
			["MadeBy"] = madeBy
		})
	end
	if bagSize > 0 then
		return bagSize
	else
		self.db:set({ace.char.id}, bagIndex)
		return 0
	end
end
function MyBagsCacheClass:SaveItemInfo(bagIndex, itemIndex) -- {{{
	local itemLink = GetContainerItemLink(bagIndex, itemIndex)
	if itemLink then
		local myColor, myLink, myName
		myName, myColor, myLink = ace.GetItemInfoFromLink(itemLink)
		local texture, itemCount = GetContainerItemInfo(bagIndex, itemIndex)
		self.db:set({ace.char.id, bagIndex}, itemIndex, {
			["Name"] = myName,
			["Color"] = myColor,
			["Link"] = myLink,
			["Count"] = itemCount,
			["Texture"] = texture,
			["Soulbound"] = soulbound,
			["MadeBy"] = madeBy,
		})
	else
		self.db:set({ace.char.id, bagIndex}, itemIndex)
	end
end
function MyBagsCacheClass:SaveEquipmentInfo(itemIndex) -- {{{
	local itemLink = GetInventoryItemLink("player",itemIndex)
	if itemLink or itemIndex == 0 then
		local myColor, myLink, myName, soulbound, madeBy = nil
		local texture = GetInventoryItemTexture("player",itemIndex)
		local count = GetInventoryItemCount("player",itemIndex)
		if itemIndex == 0 then	--AmmoSlot special case
			if texture then
				myName, myColor = MyBagsCacheClass:GetAmmoSlotTooltipInfo()
			else
				self.db:set({ace.char.id, "equipment"}, 0, nil)
			end
		else 
			myName, myColor, myLink = ace.GetItemInfoFromLink(itemLink)
		end
		self.db:set({ace.char.id, "equipment"}, itemIndex, {
			["Name"] = myName,
			["Color"] = myColor,
			["Link"] = myLink,
			["Count"] = count,
			["Texture"] = texture,
			["Soulbound"] = soulbound,
			["MadeBy"] = madeBy,
		})
	else
		self.db:set({ace.char.id, "equipment"}, itemIndex)
	end
end

MyBagsCache = MyBagsCacheClass:new()
MyBagsCache:RegisterForLoad()
